# GitFleet Architecture

This page provides an overview of GitFleet's architecture, explaining the design decisions, component interactions, and implementation details.

## High-Level Architecture

GitFleet is built with a hybrid architecture that combines the performance of Rust for critical Git operations with the flexibility and ecosystem of Python for API clients and utilities:

```
┌────────────────────────────────────────────────────────────┐
│                      Python Layer                          │
│                                                            │
│  ┌───────────────┐   ┌────────────────┐   ┌─────────────┐  │
│  │ High-level    │   │ Provider APIs  │   │ Utility     │  │
│  │ Interfaces    │   │ (GitHub, etc.) │   │ Functions   │  │
│  └───────────────┘   └────────────────┘   └─────────────┘  │
└────────────────────────────────────────────────────────────┘
                          │
                          │ PyO3
                          ▼
┌────────────────────────────────────────────────────────────┐
│                       Rust Layer                           │
│                                                            │
│  ┌───────────────┐   ┌────────────────┐   ┌─────────────┐  │
│  │ Clone         │   │ Blame & Commit │   │ Repository  │  │
│  │ Operations    │   │ Analysis       │   │ Management  │  │
│  └───────────────┘   └────────────────┘   └─────────────┘  │
└────────────────────────────────────────────────────────────┘
                          │
                          │ FFI (git2-rs)
                          ▼
┌────────────────────────────────────────────────────────────┐
│                     External Resources                      │
│                                                            │
│  ┌───────────────┐   ┌────────────────┐   ┌─────────────┐  │
│  │ Local Git     │   │ GitHub API     │   │ File System │  │
│  │ Repositories  │   │                │   │             │  │
│  └───────────────┘   └────────────────┘   └─────────────┘  │
└────────────────────────────────────────────────────────────┘
```

## Core Components

### Rust Core (Performance-Critical Operations)

The Rust layer implements performance-critical Git operations:

- **Repository Cloning**: Asynchronous cloning with progress tracking
- **Blame Analysis**: High-performance blame extraction 
- **Commit Extraction**: Efficient commit history processing
- **Repository Management**: Core repository operations

#### Key Rust Modules

- `src/lib.rs`: Main entry point and Python binding definitions
- `src/repo.rs`: Repository management logic
- `src/blame.rs`: Git blame implementation
- `src/commits.rs`: Commit history extraction
- `src/clone.rs`: Repository cloning with progress tracking

### Python Layer (API Clients & Utilities)

The Python layer provides high-level APIs, provider clients, and utilities:

- **Provider APIs**: Interfaces for Git hosting providers like GitHub
- **Data Models**: Pydantic models for validation and serialization
- **Authentication**: Token management and credential handling
- **Utility Functions**: Conversion, rate limiting, and helpers

#### Key Python Modules

- `GitFleet/__init__.py`: Main package exports
- `GitFleet/providers/github.py`: GitHub API client
- `GitFleet/providers/token_manager.py`: API token management
- `GitFleet/models/repo.py`: Pydantic wrappers for Rust objects
- `GitFleet/models/common.py`: Common data models
- `GitFleet/utils/auth.py`: Authentication utilities
- `GitFleet/utils/converters.py`: Data conversion utilities
- `GitFleet/utils/rate_limit.py`: Rate limiting helpers

## Implementation Split

The implementation responsibilities are split between Rust and Python:

### Implemented in Rust

- **Repository Cloning**: Efficient Git clone operations with progress reporting
- **Blame Analysis**: High-performance file blame extraction
- **Commit Extraction**: Fast commit history analysis
- **Core Repository Management**: Basic repository operations

### Implemented in Python

- **GitHub API Client**: Async HTTP interface to GitHub API
- **Token Management**: Smart token rotation and rate limit handling
- **Data Validation**: Pydantic models for request/response validation
- **Authentication**: Credential management and storage
- **Utilities**: Conversion to pandas DataFrames, data formatting

## Python-Rust Bridge

The Python-Rust bridge is implemented using PyO3:

- **Exposed Classes**: `RepoManager`, `CloneStatus`, `CloneTask`
- **Async Bridge**: Integration between Tokio and asyncio
- **Memory Management**: Handles reference counting between Python and Rust
- **Error Propagation**: Converts Rust errors to Python exceptions

## Concurrency Model

GitFleet uses a hybrid concurrency model:

- **Rust**: Uses Tokio for asynchronous operations
- **Python**: Exposes asyncio coroutines for non-blocking operations
- **Bridge**: Uses pyo3-async-runtimes to bridge between Tokio and asyncio

Git operations that could block (like cloning large repositories) are executed asynchronously to avoid blocking the main thread.

## Memory Management

GitFleet optimizes memory usage through:

- **Rust Ownership**: Ensures memory safety without garbage collection
- **Arc Sharing**: Shared resources use atomic reference counting
- **Temporary Directories**: Automatic cleanup of temporary clone directories
- **Stream Processing**: Large result sets can be processed as streams

## Error Handling

Error handling follows a consistent pattern:

- **Rust Results**: Functions return `Result<T, E>` for error handling
- **Python Exceptions**: Rust errors are converted to appropriate Python exceptions
- **Status Objects**: Operations provide status objects with detailed information
- **Structured Errors**: Error types are well-defined and informative

## Performance Optimizations

Several performance optimizations are employed:

- **Native Git Operations**: Critical Git operations are implemented in Rust
- **Parallel Processing**: Multiple repositories processed concurrently
- **Async I/O**: Non-blocking I/O operations for network and file system
- **Tokio Runtime**: Efficient task scheduler for concurrent operations
- **PyO3 Zero-Copy**: Minimizes data copying between Python and Rust

## Security Considerations

GitFleet prioritizes security:

- **Token Management**: Secure handling of API tokens
- **Temporary Storage**: Secure creation and cleanup of temporary directories
- **Input Validation**: Comprehensive validation of all inputs
- **Safe Defaults**: Conservative defaults for all operations

## Related Documentation

- [Python-Rust Bridge](python-rust-bridge.md): Details on the PyO3 integration
- [Contributing Guide](contributing.md): How to contribute to GitFleet
- [Performance Tips](../advanced/performance.md): Tips for maximizing performance